@@ -4,6 +4,8 @@ Plans that might be useful at the APS when using BlueSky
 .. autosummary::
    
    ~nscan
+   ~ProcedureRegistry
+   ~run_blocker_in_plan
    ~run_in_thread
    ~TuneAxis
    ~tune_axes
@@ -17,11 +19,13 @@ import datetime
 import logging
 import numpy as np 
 import threading
+import time
 
 from bluesky import preprocessors as bpp
 from bluesky import plan_stubs as bps
 from bluesky import plans as bp
 from bluesky.callbacks.fitting import PeakStats
+import ophyd
 
 
 logger = logging.getLogger(__name__).addHandler(logging.NullHandler())
@@ -50,6 +54,50 @@ def run_in_thread(func):
     return wrapper
 
 
+def run_blocker_in_plan(blocker, *args, _poll_s_=0.01, _timeout_s_=None, **kwargs):
+    """
+    run blocking function ``blocker_(*args, **kwargs)`` from a Bluesky plan
+    
+    blocker (func) : function object to be called in a Bluesky plan
+
+    _poll_s_ (float) : sleep interval in loop while waiting for completion (default: 0.01)
+
+    _timeout_s_ (float) : maximum time for completion (default: `None` which means no timeout)
+    
+    Example (using ``time.sleep`` as blocking function)::
+    
+        RE(run_blocker_in_plan(time.sleep, 2.14))
+
+    Example (within a plan, using ``time.sleep`` as blocking function)::
+
+        def my_sleep(t=1.0):
+            yield from run_blocker_in_plan(time.sleep, t)
+
+        RE(my_sleep())
+
+    """
+    status = ophyd.status.Status()
+    
+    @run_in_thread
+    def _internal(blocking_function, *args, **kwargs):
+        blocking_function(*args, **kwargs)
+        status._finished(success=True, done=True)
+    
+    if _timeout_s_ is not None:
+        t_expire = time.time() + _timeout_s_
+
+    # FIXME: how to keep this from running during summarize_plan()?
+    _internal(blocker, *args, **kwargs)
+
+    while not status.done:
+        if _timeout_s_ is not None:
+            if time.time() > t_expire:
+                status._finished(success=False, done=True)
+                break
+        yield from bps.sleep(_poll_s_)
+    return status
+
+
 def nscan(detectors, *motor_sets, num=11, per_step=None, md=None):
     """
     Scan over ``n`` variables moved together, each in equally spaced steps.
@@ -349,3 +397,140 @@ def tune_axes(axes):
     """
     for axis in axes:
         yield from axis.tune()
+
+
+class ProcedureRegistry(ophyd.Device):
+    """
+    Procedure Registry
+    
+    With many instruments, such as USAXS,, there are several operating 
+    modes to be used, each with its own setup code.  This ophyd Device
+    should coordinate those modes so that the setup procedures can be called
+    either as part of a Bluesky plan or from the command line directly.
+
+    Assumes that users will write functions to setup a particular 
+    operation or operating mode.  The user-written functions may not
+    be appropriate to use in a plan directly since they might
+    make blocking calls.  The ProcedureRegistry will call the function
+    in a thread (which is allowed to make blocking calls) and wait
+    for the thread to complete.
+    
+    It is assumed that each user-written function will not return until
+    it is complete.
+
+    .. autosummary::
+       
+        ~dir
+        ~add
+        ~remove
+        ~set
+        ~put
+
+    EXAMPLE::
+
+        use_mode = ProcedureRegistry(name="use_mode")
+
+        def clearScalerNames():
+            for ch in scaler.channels.configuration_attrs:
+                if ch.find(".") < 0:
+                    chan = scaler.channels.__getattribute__(ch)
+                    chan.chname.put("")
+
+        def setMyScalerNames():
+            scaler.channels.chan01.chname.put("clock")
+            scaler.channels.chan02.chname.put("I0")
+            scaler.channels.chan03.chname.put("detector")
+
+        def useMyScalerNames(): # Bluesky plan
+            yield from bps.mv(
+                m1, 5,
+                use_mode, "clear",
+            )
+            yield from bps.mv(
+                m1, 0,
+                use_mode, "set",
+            )
+
+        def demo():
+            print(1)
+            m1.move(5)
+            print(2)
+            time.sleep(2)
+            print(3)
+            m1.move(0)
+            print(4)
+
+
+        use_mode.add(demo)
+        use_mode.add(clearScalerNames, "clear")
+        use_mode.add(setMyScalerNames, "set")
+        # use_mode.set("demo")
+        # use_mode.set("clear")
+        # RE(useMyScalerNames())
+
+    """
+    
+    busy = ophyd.Component(ophyd.Signal, value=False)
+    registry = {}
+    delay_s = 0
+    timeout_s = None
+    state = "__created__"
+    
+    @property
+    def dir(self):
+        """tuple of procedure names"""
+        return tuple(sorted(self.registry.keys()))
+    
+    def add(self, procedure, proc_name=None):
+        """
+        add procedure to registry
+        """
+        #if procedure.__class__ == "function":
+        nm = proc_name or procedure.__name__
+        self.registry[nm] = procedure
+    
+    def remove(self, procedure):
+        """
+        remove procedure from registry
+        """
+        if isinstance(procedure, str):
+            nm = procedure
+        else:
+            nm = procedure.__name__
+        if nm in self.registry:
+            del self.registry[nm]
+    
+    def set(self, proc_name):
+        """
+        run procedure in a thread, return once it is complete
+        
+        proc_name (str) : name of registered procedure to be run
+        """
+        if not isinstance(proc_name, str):
+            raise ValueError("expected a procedure name, not {}".format(proc_name))
+        if proc_name not in self.registry:
+            raise KeyError("unknown procedure name: "+proc_name)
+        if self.busy.value:
+            raise RuntimeError("busy now")
+ 
+        self.state = "__busy__"
+        status = ophyd.DeviceStatus(self)
+        
+        @run_in_thread
+        def run_and_delay():
+            self.busy.put(True)
+            self.registry[proc_name]()
+            # optional delay
+            if self.delay_s > 0:
+                time.sleep(self.delay_s)
+            self.busy.put(False)
+            status._finished(success=True)
+        
+        run_and_delay()
+        ophyd.status.wait(status, timeout=self.timeout_s)
+        self.state = proc_name
+        return status
+    
+    def put(self, value):   # TODO: risky?
+        """replaces ophyd Device default put() behavior"""
+        self.set(value)
